unit DigitalBank;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, OleCtrls, ExtCtrls, OptoSnapIoMemMapXLib_TLB;

type
  TForm1 = class(TForm)
    GroupBox1: TGroupBox;
    Label1: TLabel;
    IpAddressEdit: TEdit;
    Label2: TLabel;
    PortNumberEdit: TEdit;
    ConnectBtn: TButton;
    DisconnectBtn: TButton;
    GroupBox2: TGroupBox;
    ErrorEdit: TEdit;
    GroupBox3: TGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    BankStateEdit: TEdit;
    OnLatchStateEdit: TEdit;
    OffLatchStateEdit: TEdit;
    Button1: TButton;
    Button2: TButton;
    GroupBox4: TGroupBox;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    TurnOnEdit: TEdit;
    TurnOffEdit: TEdit;
    SetStatesEdit: TEdit;
    Button3: TButton;
    Button4: TButton;
    Button5: TButton;
    Timer1: TTimer;
    Brain: TOptoSnapIoMemMapX;
    procedure ConnectBtnClick(Sender: TObject);
    procedure DisconnectBtnClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure HandleError(nResult: Integer);
    function  GetResultAsString(nResult: Integer): String;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button5Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }

  end;

var
  Form1: TForm1;
//  Brain: TROptoSnapIoMemMapX;

const SIOMM_OK = 1;
const SIOMM_ERROR = -1;
const SIOMM_TIME_OUT = -2;
const SIOMM_ERROR_NO_SOCKETS = -3;
const SIOMM_ERROR_CREATING_SOCKET = -4;
const SIOMM_ERROR_CONNECTING_SOCKET = -5;
const SIOMM_ERROR_RESPONSE_BAD = -6;
const SIOMM_ERROR_NOT_CONNECTED_YET = -7;
const SIOMM_ERROR_OUT_OF_MEMORY = -8;
const SIOMM_ERROR_NOT_CONNECTED = -9;
const SIOMM_BRAIN_ERROR_UNDEFINED_CMD = 57345;
const SIOMM_BRAIN_ERROR_INVALID_PT_TYPE = 57346;
const SIOMM_BRAIN_ERROR_INVALID_FLOAT = 57347;
const SIOMM_BRAIN_ERROR_PUC_EXPECTED = 57348;
const SIOMM_BRAIN_ERROR_INVALID_ADDRESS = 57349;
const SIOMM_BRAIN_ERROR_INVALID_CMD_LENGTH = 57350;
const SIOMM_BRAIN_ERROR_RESERVED = 57351;
const SIOMM_BRAIN_ERROR_BUSY = 57352;
const SIOMM_BRAIN_ERROR_CANT_ERASE_FLASH = 57353;
const SIOMM_BRAIN_ERROR_CANT_PROG_FLASH = 57354;
const SIOMM_BRAIN_ERROR_IMAGE_TOO_SMALL = 57355;
const SIOMM_BRAIN_ERROR_IMAGE_CRC_MISMATCH = 57356;
const SIOMM_BRAIN_ERROR_IMAGE_LEN_MISMATCH = 57357;

implementation

{$R *.DFM}


procedure TForm1.ConnectBtnClick(Sender: TObject);

  var
    nResult: Integer;

  begin

    // Attempt to open the Snap I/O..
    nResult := Brain.OpenEnet(IpAddressEdit.Text, StrToInt(PortNumberEdit.Text), 3000, 1);

  // Check the result
  if (nResult = SIOMM_OK) then
  begin
    Form1.Cursor := crHourglass;
    Form1.Update();

    // Sleep for a little while
    Sleep(250);

    // Keep calling IsOpenDone() until we succeed, time-out, or error
    nResult := Brain.IsOpenDone();
    while (nResult = SIOMM_ERROR_NOT_CONNECTED_YET) do
    begin
      // let other tasks get some CPU
      Sleep(250);

      // Check the status of the open operation
      nResult := Brain.IsOpenDone(); // keep trying
    end;

    // Reset the cursor to the arrow
    Form1.Cursor := crArrow;
  end;

  // Check the result
  HandleError(nResult);

  // If no error, then enable the timer
  if (nResult = SIOMM_OK) then
  begin
    Timer1.Enabled := True;
  end;
end;

procedure TForm1.DisconnectBtnClick(Sender: TObject);
var
  nResult: Integer;
begin
  Timer1.Enabled := False;

  nResult := Brain.Close();

  if (nResult = SIOMM_OK) then
    // Even though the result was OK, show the Not Connected error
    HandleError(SIOMM_ERROR_NOT_CONNECTED)
  else
    HandleError(nResult);

end;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  nResult: Integer;
  nStatePts63to32: Integer;
  nStatePts31to0: Integer;
  nOnLatchStatePts63to32: Integer;
  nOnLatchStatePts31to0: Integer;
  nOffLatchStatePts63to32: Integer;
  nOffLatchStatePts31to0: Integer;
  nActiveCountersPts63to32: Integer;
  nActiveCountersPts31to0: Integer;

begin


  // Get the digital bank read area
  nResult := Brain.GetDigBankReadArea(nStatePts63to32,          nStatePts31to0,
                                      nOnLatchStatePts63to32,   nOnLatchStatePts31to0,
                                      nOffLatchStatePts63to32,  nOffLatchStatePts31to0,
                                      nActiveCountersPts63to32, nActiveCountersPts31to0);

  if (nResult = SIOMM_OK) then
  begin
    // If no error, update all the bitmasks
    BankStateEdit.Text     := Format('0x%.8x%.8x', [nStatePts63to32,         nStatePts31to0]);
    OnLatchStateEdit.Text  := Format('0x%.8x%.8x', [nOnLatchStatePts63to32,  nOnLatchStatePts31to0]);
    OffLatchStateEdit.Text := Format('0x%.8x%.8x', [nOffLatchStatePts63to32, nOffLatchStatePts31to0]);
  end
  else
  begin
    // If error, clear out all the bitmasks
    BankStateEdit.Text     := '';
    OnLatchStateEdit.Text  := '';
    OffLatchStateEdit.Text := '';
  end;

  // Handle any errors
  HandleError(nResult);

end;

procedure TForm1.HandleError(nResult: Integer);
begin
  // Update the error message
  ErrorEdit.Text := GetResultAsString(nResult);

  if (nResult = SIOMM_OK) then
    // Set background to white
    ErrorEdit.Color := $FFFFFF
  else
  begin
    // Set background to red
    ErrorEdit.Color := $FF;

    // shutdown everything on a bad error
    if (nResult <> SIOMM_ERROR_NOT_CONNECTED_YET) then
    begin
      Brain.Close;
      Timer1.Enabled := False;
    end;
  end;
end;

function TForm1.GetResultAsString(nResult: Integer): String;
begin
    If (nResult = SIOMM_OK) Then
      Result := 'Ok'
    else if (nResult = SIOMM_ERROR) Then
      Result := 'General error'
    else if (nResult = SIOMM_TIME_OUT) Then
      Result := 'Timeout'
    else if (nResult = SIOMM_ERROR_NO_SOCKETS) Then
      Result := 'No sockets'
    else if (nResult = SIOMM_ERROR_CREATING_SOCKET) Then
      Result := 'Could not create socket to Snap I/O'
    else if (nResult = SIOMM_ERROR_CONNECTING_SOCKET) Then
      GetResultAsString := 'Could not connect socket to Snap I/O'
    else if (nResult = SIOMM_ERROR_RESPONSE_BAD) Then
      Result := 'Bad response'
    else if (nResult = SIOMM_ERROR_NOT_CONNECTED_YET) Then
      Result := 'Not connected yet'
    else if (nResult = SIOMM_ERROR_OUT_OF_MEMORY) Then
      Result := 'ActiveX out of memory'
    else if (nResult = SIOMM_ERROR_NOT_CONNECTED) Then
      Result := 'Not connected'
    else if (nResult = SIOMM_BRAIN_ERROR_UNDEFINED_CMD) Then
      Result := 'Snap I/O Error: Undefined Command'
    else if (nResult = SIOMM_BRAIN_ERROR_INVALID_PT_TYPE) Then
      Result := 'Snap I/O Error: Invalid point type'
    else if (nResult = SIOMM_BRAIN_ERROR_INVALID_FLOAT) Then
      Result := 'Snap I/O Error: Invalid float'
    else if (nResult = SIOMM_BRAIN_ERROR_PUC_EXPECTED) Then
      Result := 'Snap I/O Error: Powerup Clear expected'
    else if (nResult = SIOMM_BRAIN_ERROR_INVALID_ADDRESS) Then
      Result := 'Snap I/O Error: Invalid memory address'
    else if (nResult = SIOMM_BRAIN_ERROR_INVALID_CMD_LENGTH) Then
      Result := 'Snap I/O Error: Invalid command length'
    else if (nResult = SIOMM_BRAIN_ERROR_RESERVED) Then
      Result := 'Snap I/O Error: Reserved'
    else if (nResult = SIOMM_BRAIN_ERROR_BUSY) Then
      Result := 'Snap I/O Error: Busy'
    else if (nResult = SIOMM_BRAIN_ERROR_CANT_ERASE_FLASH) Then
      Result := 'Snap I/O Error: Cannot erase flash'
    else if (nResult = SIOMM_BRAIN_ERROR_CANT_PROG_FLASH) Then
      Result := 'Snap I/O Error: Cannot program flash'
    else if (nResult = SIOMM_BRAIN_ERROR_IMAGE_TOO_SMALL) Then
      Result := 'Snap I/O Error: Downloaded imaged too small'
    else if (nResult = SIOMM_BRAIN_ERROR_IMAGE_CRC_MISMATCH) Then
      Result := 'Snap I/O Error: Image CRC mismatch'
    else if (nResult = SIOMM_BRAIN_ERROR_IMAGE_LEN_MISMATCH) Then
      Result := 'Snap I/O Error: Image length mismatch'
    else
      Result := 'Unknown error'

end; // GetResultAsString()

procedure TForm1.Button1Click(Sender: TObject);
var
  nPoint: Integer;
  nResult: Integer;
  nState: Integer;
begin

  // Clear the on-latches by reading and clearing them all.
  For nPoint := 0 To 63 Do
  begin
    nResult := Brain.ReadClearDigPtOnLatch(nPoint, nState);
  end;

  HandleError(nResult);

end;

procedure TForm1.Button2Click(Sender: TObject);
var
  nPoint: Integer;
  nResult: Integer;
  nState: Integer;
begin

  // Clear the on-latches by reading and clearing them all.
  For nPoint := 0 To 63 Do
  begin
    nResult := Brain.ReadClearDigPtOffLatch(nPoint, nState);
  end;

  HandleError(nResult);

end;

procedure TForm1.Button3Click(Sender: TObject);
var
  n63to32Part: Integer;
  n31to0Part: Integer;
  nResult: Integer;
begin

  // Convert the string into a bitmask
  n63to32Part := StrToInt('0x' + Copy(TurnOnEdit.Text, 3, 8));
  n31to0Part  := StrToInt('0x' + Copy(TurnOnEdit.Text, 11, 8));

  // Set the on mask
  nResult := Brain.SetDigBankOnMask(n63to32Part, n31to0Part);

  // Handle any errors
  HandleError(nResult);

end;

procedure TForm1.Button4Click(Sender: TObject);
var
  n63to32Part: Integer;
  n31to0Part: Integer;
  nResult: Integer;
begin
  // Convert the string into a bitmask
  n63to32Part := StrToInt('0x' + Copy(TurnOffEdit.Text, 3, 8));
  n31to0Part  := StrToInt('0x' + Copy(TurnOffEdit.Text, 11, 8));

  // Set the on mask
  nResult := Brain.SetDigBankOffMask(n63to32Part, n31to0Part);

  // Handle any errors
  HandleError(nResult);
end;


procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Timer1.Enabled := False;
  Brain.Close();
end;

procedure TForm1.Button5Click(Sender: TObject);
var
  n63to32Part: Integer;
  n31to0Part: Integer;
  nResult: Integer;
begin
  // Convert the string into a bitmask
  n63to32Part := StrToInt('0x' + Copy(SetStatesEdit.Text, 3, 8));
  n31to0Part  := StrToInt('0x' + Copy(SetStatesEdit.Text, 11, 8));

  // Set the on mask
  nResult := Brain.SetDigBankPointStates(n63to32Part, n31to0Part, -1, -1);

  // Handle any errors
  HandleError(nResult);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  
  // Show that there is no connection
  HandleError(SIOMM_ERROR_NOT_CONNECTED);
end;

end.

