Option Strict Off
Option Explicit On
Module OptoSnapIoMemMapX_VB_Utilities
	Private Declare Sub Sleep Lib "kernel32" (ByVal N As Short)
	
	' Result codes of the ActiveX component
	Public Const SIOMM_OK As Integer = 1
	Public Const SIOMM_ERROR As Short = -1
	Public Const SIOMM_TIME_OUT As Short = -2
	Public Const SIOMM_ERROR_NO_SOCKETS As Short = -3
	Public Const SIOMM_ERROR_CREATING_SOCKET As Short = -4
	Public Const SIOMM_ERROR_CONNECTING_SOCKET As Short = -5
	Public Const SIOMM_ERROR_RESPONSE_BAD As Short = -6
	Public Const SIOMM_ERROR_NOT_CONNECTED_YET As Short = -7
	Public Const SIOMM_ERROR_OUT_OF_MEMORY As Short = -8
	Public Const SIOMM_ERROR_NOT_CONNECTED As Short = -9
	
	' Result codes from the SNAP brain
	Public Const SIOMM_BRAIN_ERROR_UNDEFINED_CMD As Integer = 57345
	Public Const SIOMM_BRAIN_ERROR_INVALID_PT_TYPE As Integer = 57346
	Public Const SIOMM_BRAIN_ERROR_INVALID_FLOAT As Integer = 57347
	Public Const SIOMM_BRAIN_ERROR_PUC_EXPECTED As Integer = 57348
	Public Const SIOMM_BRAIN_ERROR_INVALID_ADDRESS As Integer = 57349
	Public Const SIOMM_BRAIN_ERROR_INVALID_CMD_LENGTH As Integer = 57350
	Public Const SIOMM_BRAIN_ERROR_RESERVED As Integer = 57351
	Public Const SIOMM_BRAIN_ERROR_BUSY As Integer = 57352
	Public Const SIOMM_BRAIN_ERROR_CANT_ERASE_FLASH As Integer = 57353
	Public Const SIOMM_BRAIN_ERROR_CANT_PROG_FLASH As Integer = 57354
	Public Const SIOMM_BRAIN_ERROR_IMAGE_TOO_SMALL As Integer = 57355
	Public Const SIOMM_BRAIN_ERROR_IMAGE_CRC_MISMATCH As Integer = 57356
	Public Const SIOMM_BRAIN_ERROR_IMAGE_LEN_MISMATCH As Integer = 57357
	
	
	' Connection types for OptoEnet2()
	Public Const SIOMM_TCP As Short = 1
	Public Const SIOMM_UDP As Short = 2
	
	
	Public Function FormatHexString(ByRef nNum As Integer) As String
		FormatHexString = "00000000" & Hex(nNum)
		FormatHexString = Right(FormatHexString, 8)
	End Function
	
	
	Public Function GetBitmask64FromString(ByRef strBitmask As String, ByRef n63to32Part As Integer, ByRef n31to0Part As Integer) As Integer
		Dim strTemp As String
		
		strTemp = Right(strBitmask, 8)
		strTemp = "&H" & strTemp
		n31to0Part = CInt(strTemp)
		
		strTemp = Mid(strBitmask, 3, 8)
		strTemp = "&H" & strTemp
		n63to32Part = CInt(strTemp)
		
		GetBitmask64FromString = True
	End Function
	
	
	Public Function GetModuleTypeAsString(ByRef nModuleType As Integer) As String
		If (nModuleType = 0) Then
			GetModuleTypeAsString = "Digital or empty"
		ElseIf (nModuleType = &H4) Then 
			GetModuleTypeAsString = "AICTD"
		ElseIf (nModuleType = &H9) Then 
			GetModuleTypeAsString = "AITM-2"
		ElseIf (nModuleType = &H10) Then 
			GetModuleTypeAsString = "AIRTD"
		ElseIf (nModuleType = &H12) Then 
			GetModuleTypeAsString = "AIV"
		ElseIf (nModuleType = &H64) Then 
			GetModuleTypeAsString = "AIMA"
		ElseIf (nModuleType = &H66) Then 
			GetModuleTypeAsString = "AITM"
		ElseIf (nModuleType = &H69) Then 
			GetModuleTypeAsString = "AIRATE"
		ElseIf (nModuleType = &H70) Then 
			GetModuleTypeAsString = "AIVRMS"
		ElseIf (nModuleType = &H71) Then 
			GetModuleTypeAsString = "AIARMS"
		ElseIf (nModuleType = &HA3) Then 
			GetModuleTypeAsString = "AOA23"
		ElseIf (nModuleType = &HA5) Then 
			GetModuleTypeAsString = "AOV25"
		ElseIf (nModuleType = &HA7) Then 
			GetModuleTypeAsString = "AOV27"
		ElseIf (nModuleType = &HA8) Then 
			GetModuleTypeAsString = "AOV28"
		Else
			GetModuleTypeAsString = "Unknown Module"
		End If
	End Function
	
	Public Function GetFeatureAsString(ByRef nModuleType As Integer) As String
		If (nModuleType = 0) Then
			GetFeatureAsString = "None"
		ElseIf (nModuleType = 1) Then 
			GetFeatureAsString = "Counter"
		Else
			GetFeatureAsString = "Unknown Feature"
		End If
	End Function
	
	Public Function GetVersionAsString(ByRef nVersion As Integer) As String
		Dim TempStr As String
		
		' Since VisualBasic doesn't provide bit-shifting, we have to do some
		' pretty messy string manipulation:
		' 1. Convert the version to a hex string by using Hex()
		' 2. Make sure the hex string is 8 characters by using Right()
		' 3. Extract the byte we're interested in by using Left() and Mid()
		' 4. Convert the hex byte string into a decimal string by using Val() and Str()
		TempStr = VB6.Format(Val("&H" & Left(Right("00000000" & Hex(nVersion), 8), 2)))
		TempStr = TempStr & "." & VB6.Format(Val("&H" & Mid(Right("00000000" & Hex(nVersion), 8), 3, 2)))
		TempStr = TempStr & "." & VB6.Format(Val("&H" & Mid(Right("00000000" & Hex(nVersion), 8), 5, 2)))
		TempStr = TempStr & "." & VB6.Format(Val("&H" & Mid(Right("00000000" & Hex(nVersion), 8), 7, 2)))
		
		GetVersionAsString = TempStr
	End Function
	
	
	Public Function GetMACAddressAsString(ByRef wMACAddress0 As Short, ByRef wMACAddress1 As Short, ByRef wMACAddress2 As Short) As String
		Dim TempStr As String
		
		TempStr = Right("0000" & Hex(wMACAddress0), 4)
		GetMACAddressAsString = Left(TempStr, 2) & "-" & Right(TempStr, 2) & "-"
		
		TempStr = Right("0000" & Hex(wMACAddress1), 4)
		GetMACAddressAsString = GetMACAddressAsString & Left(TempStr, 2) & "-" & Right(TempStr, 2) & "-"
		
		TempStr = Right("0000" & Hex(wMACAddress2), 4)
		GetMACAddressAsString = GetMACAddressAsString & Left(TempStr, 2) & "-" & Right(TempStr, 2)
		
	End Function
	
	
	Public Function GetIPAddressAsString(ByRef nIpAddress As Integer) As String
		Dim TempStr As String
		
		' Since VisualBasic doesn't provide bit-shifting, we have to do some
		' pretty messy string manipulation:
		' 1. Convert the version to a hex string by using Hex()
		' 2. Make sure the hex string is 8 characters by using Right()
		' 3. Extract the byte we're interested in by using Left() and Mid()
		' 4. Convert the hex byte string into a decimal string by using Val() and Str()
		TempStr = VB6.Format(Val("&H" & Left(Right("00000000" & Hex(nIpAddress), 8), 2)))
		TempStr = TempStr & "." & VB6.Format(Val("&H" & Mid(Right("00000000" & Hex(nIpAddress), 8), 3, 2)))
		TempStr = TempStr & "." & VB6.Format(Val("&H" & Mid(Right("00000000" & Hex(nIpAddress), 8), 5, 2)))
		TempStr = TempStr & "." & VB6.Format(Val("&H" & Mid(Right("00000000" & Hex(nIpAddress), 8), 7, 2)))
		
		GetIPAddressAsString = TempStr
	End Function
	
	
	Public Function GetResultAsString(ByRef nResult As Integer) As String
		
		If (nResult = SIOMM_OK) Then
			GetResultAsString = "Ok"
		ElseIf (nResult = SIOMM_ERROR) Then 
			GetResultAsString = "General error"
		ElseIf (nResult = SIOMM_TIME_OUT) Then 
			GetResultAsString = "Timeout"
		ElseIf (nResult = SIOMM_ERROR_NO_SOCKETS) Then 
			GetResultAsString = "No sockets"
		ElseIf (nResult = SIOMM_ERROR_CREATING_SOCKET) Then 
			GetResultAsString = "Could not create socket to Snap I/O"
		ElseIf (nResult = SIOMM_ERROR_CONNECTING_SOCKET) Then 
			GetResultAsString = "Could not connect socket to Snap I/O"
		ElseIf (nResult = SIOMM_ERROR_RESPONSE_BAD) Then 
			GetResultAsString = "Bad response"
		ElseIf (nResult = SIOMM_ERROR_NOT_CONNECTED_YET) Then 
			GetResultAsString = "Not connected yet"
		ElseIf (nResult = SIOMM_ERROR_OUT_OF_MEMORY) Then 
			GetResultAsString = "ActiveX out of memory"
		ElseIf (nResult = SIOMM_ERROR_NOT_CONNECTED) Then 
			GetResultAsString = "Not connected"
		ElseIf (nResult = SIOMM_BRAIN_ERROR_UNDEFINED_CMD) Then 
			GetResultAsString = "Snap I/O Error: Undefined Command"
		ElseIf (nResult = SIOMM_BRAIN_ERROR_INVALID_PT_TYPE) Then 
			GetResultAsString = "Snap I/O Error: Invalid point type"
		ElseIf (nResult = SIOMM_BRAIN_ERROR_INVALID_FLOAT) Then 
			GetResultAsString = "Snap I/O Error: Invalid float"
		ElseIf (nResult = SIOMM_BRAIN_ERROR_PUC_EXPECTED) Then 
			GetResultAsString = "Snap I/O Error: Powerup Clear expected"
		ElseIf (nResult = SIOMM_BRAIN_ERROR_INVALID_ADDRESS) Then 
			GetResultAsString = "Snap I/O Error: Invalid memory address"
		ElseIf (nResult = SIOMM_BRAIN_ERROR_INVALID_CMD_LENGTH) Then 
			GetResultAsString = "Snap I/O Error: Invalid command length"
		ElseIf (nResult = SIOMM_BRAIN_ERROR_RESERVED) Then 
			GetResultAsString = "Snap I/O Error: Reserved"
		ElseIf (nResult = SIOMM_BRAIN_ERROR_BUSY) Then 
			GetResultAsString = "Snap I/O Error: Busy"
		ElseIf (nResult = SIOMM_BRAIN_ERROR_CANT_ERASE_FLASH) Then 
			GetResultAsString = "Snap I/O Error: Cannot erase flash"
		ElseIf (nResult = SIOMM_BRAIN_ERROR_CANT_PROG_FLASH) Then 
			GetResultAsString = "Snap I/O Error: Cannot program flash"
		ElseIf (nResult = SIOMM_BRAIN_ERROR_IMAGE_TOO_SMALL) Then 
			GetResultAsString = "Snap I/O Error: Downloaded imaged too small"
		ElseIf (nResult = SIOMM_BRAIN_ERROR_IMAGE_CRC_MISMATCH) Then 
			GetResultAsString = "Snap I/O Error: Image CRC mismatch"
		ElseIf (nResult = SIOMM_BRAIN_ERROR_IMAGE_LEN_MISMATCH) Then 
			GetResultAsString = "Snap I/O Error: Image length mismatch"
		Else
			GetResultAsString = "Unknown error"
		End If
		
	End Function
	
	Public Function OpenSnapIoWithCancel(ByRef Brain As OptoSnapIoMemMapXLib.OptoSnapIoMemMapX, ByRef IpAddress As String, ByRef PortNumber As Short, ByRef Timeout As Integer) As Integer
		Dim nResult As Integer
		
		' Attempt to open the Snap I/O..
		nResult = Brain.OpenEnet2(IpAddress, PortNumber, Timeout, 1, SIOMM_TCP)
		
		' Check the result
		If (nResult = SIOMM_OK) Then
			
			' Sleep for a little while
			Sleep((250))
			
			' Set the cancel flag to false
			dlgOpenSnapIo.DefInstance.bCancel = False
			
			' Keep calling IsOpenDone() until we succeed, time-out, or error
			nResult = Brain.IsOpenDone()
			While ((nResult = SIOMM_ERROR_NOT_CONNECTED_YET) And (Not dlgOpenSnapIo.DefInstance.bCancel))
				
				' let other tasks get some CPU
				System.Windows.Forms.Application.DoEvents()
				
				' Show the cancel dialog
				VB6.ShowForm(dlgOpenSnapIo.DefInstance, (0))
				
				' Check the status of the open operation
				nResult = Brain.IsOpenDone() ' keep trying
			End While
			
			' hide the cancel box
			Call dlgOpenSnapIo.DefInstance.Hide()
			
			' check if we should close the B3000
			If (dlgOpenSnapIo.DefInstance.bCancel = True) Then
				Call Brain.Close()
			End If
		End If
		
		OpenSnapIoWithCancel = nResult
		
	End Function
	
	Public Function IsBitTrue(ByRef nNum As Integer, ByRef nBit As Integer) As Boolean
		If (nBit = 31) Then
			IsBitTrue = (nNum < 0)
		Else
			IsBitTrue = nNum And (2 ^ nBit)
		End If
	End Function
End Module