Attribute VB_Name = "OptoSnapIoMemMapX_VB_Utilities"
Option Explicit
Private Declare Sub Sleep Lib "kernel32" (ByVal N As Integer)

' Result codes of the ActiveX component
Public Const SIOMM_OK As Long = 1
Public Const SIOMM_ERROR = -1
Public Const SIOMM_TIME_OUT = -2
Public Const SIOMM_ERROR_NO_SOCKETS = -3
Public Const SIOMM_ERROR_CREATING_SOCKET = -4
Public Const SIOMM_ERROR_CONNECTING_SOCKET = -5
Public Const SIOMM_ERROR_RESPONSE_BAD = -6
Public Const SIOMM_ERROR_NOT_CONNECTED_YET = -7
Public Const SIOMM_ERROR_OUT_OF_MEMORY = -8
Public Const SIOMM_ERROR_NOT_CONNECTED = -9

' Result codes from the SNAP brain
Public Const SIOMM_BRAIN_ERROR_UNDEFINED_CMD = 57345
Public Const SIOMM_BRAIN_ERROR_INVALID_PT_TYPE = 57346
Public Const SIOMM_BRAIN_ERROR_INVALID_FLOAT = 57347
Public Const SIOMM_BRAIN_ERROR_PUC_EXPECTED = 57348
Public Const SIOMM_BRAIN_ERROR_INVALID_ADDRESS = 57349
Public Const SIOMM_BRAIN_ERROR_INVALID_CMD_LENGTH = 57350
Public Const SIOMM_BRAIN_ERROR_RESERVED = 57351
Public Const SIOMM_BRAIN_ERROR_BUSY = 57352
Public Const SIOMM_BRAIN_ERROR_CANT_ERASE_FLASH = 57353
Public Const SIOMM_BRAIN_ERROR_CANT_PROG_FLASH = 57354
Public Const SIOMM_BRAIN_ERROR_IMAGE_TOO_SMALL = 57355
Public Const SIOMM_BRAIN_ERROR_IMAGE_CRC_MISMATCH = 57356
Public Const SIOMM_BRAIN_ERROR_IMAGE_LEN_MISMATCH = 57357


' Connection types for OptoEnet2()
Public Const SIOMM_TCP = 1
Public Const SIOMM_UDP = 2


Public Function FormatHexString(nNum As Long) As String
  FormatHexString = "00000000" + Hex(nNum)
  FormatHexString = Right(FormatHexString, 8)
End Function


Public Function GetBitmask64FromString(strBitmask As String, ByRef n63to32Part As Long, ByRef n31to0Part As Long) As Long
  Dim strTemp As String
  
  strTemp = Right(strBitmask, 8)
  strTemp = "&H" + strTemp
  n31to0Part = CLng(strTemp)

  strTemp = Mid(strBitmask, 3, 8)
  strTemp = "&H" + strTemp
  n63to32Part = CLng(strTemp)
  
  GetBitmask64FromString = True
End Function


Public Function GetModuleTypeAsString(nModuleType As Long) As String
  If (nModuleType = 0) Then
   GetModuleTypeAsString = "Digital or empty"
  ElseIf (nModuleType = &H4&) Then
   GetModuleTypeAsString = "AICTD"
  ElseIf (nModuleType = &H9&) Then
   GetModuleTypeAsString = "AITM-2"
  ElseIf (nModuleType = &H10&) Then
   GetModuleTypeAsString = "AIRTD"
  ElseIf (nModuleType = &H12&) Then
   GetModuleTypeAsString = "AIV"
  ElseIf (nModuleType = &H64&) Then
   GetModuleTypeAsString = "AIMA"
  ElseIf (nModuleType = &H66&) Then
   GetModuleTypeAsString = "AITM"
  ElseIf (nModuleType = &H69&) Then
   GetModuleTypeAsString = "AIRATE"
  ElseIf (nModuleType = &H70&) Then
   GetModuleTypeAsString = "AIVRMS"
  ElseIf (nModuleType = &H71&) Then
   GetModuleTypeAsString = "AIARMS"
  ElseIf (nModuleType = &HA3&) Then
   GetModuleTypeAsString = "AOA23"
  ElseIf (nModuleType = &HA5&) Then
   GetModuleTypeAsString = "AOV25"
  ElseIf (nModuleType = &HA7&) Then
   GetModuleTypeAsString = "AOV27"
  ElseIf (nModuleType = &HA8&) Then
   GetModuleTypeAsString = "AOV28"
  Else
    GetModuleTypeAsString = "Unknown Module"
  End If
End Function

Public Function GetFeatureAsString(nModuleType As Long) As String
  If (nModuleType = 0) Then
   GetFeatureAsString = "None"
  ElseIf (nModuleType = 1) Then
   GetFeatureAsString = "Counter"
  Else
    GetFeatureAsString = "Unknown Feature"
  End If
End Function

Public Function GetVersionAsString(nVersion As Long) As String
  Dim TempStr As String
  
  ' Since VisualBasic doesn't provide bit-shifting, we have to do some
  ' pretty messy string manipulation:
  ' 1. Convert the version to a hex string by using Hex()
  ' 2. Make sure the hex string is 8 characters by using Right()
  ' 3. Extract the byte we're interested in by using Left() and Mid()
  ' 4. Convert the hex byte string into a decimal string by using Val() and Str()
  TempStr = Format(Val("&H" + Left(Right("00000000" + Hex(nVersion), 8), 2)))
  TempStr = TempStr + "." + Format(Val("&H" + Mid(Right("00000000" + Hex(nVersion), 8), 3, 2)))
  TempStr = TempStr + "." + Format(Val("&H" + Mid(Right("00000000" + Hex(nVersion), 8), 5, 2)))
  TempStr = TempStr + "." + Format(Val("&H" + Mid(Right("00000000" + Hex(nVersion), 8), 7, 2)))
  
  GetVersionAsString = TempStr
End Function


Public Function GetMACAddressAsString(wMACAddress0 As Integer, wMACAddress1 As Integer, wMACAddress2 As Integer) As String
  Dim TempStr As String
  
  TempStr = Right("0000" + Hex(wMACAddress0), 4)
  GetMACAddressAsString = Left(TempStr, 2) + "-" + Right(TempStr, 2) + "-"
  
  TempStr = Right("0000" + Hex(wMACAddress1), 4)
  GetMACAddressAsString = GetMACAddressAsString + Left(TempStr, 2) + "-" + Right(TempStr, 2) + "-"
  
  TempStr = Right("0000" + Hex(wMACAddress2), 4)
  GetMACAddressAsString = GetMACAddressAsString + Left(TempStr, 2) + "-" + Right(TempStr, 2)
  
End Function


Public Function GetIPAddressAsString(nIpAddress As Long) As String
  Dim TempStr As String
  
  ' Since VisualBasic doesn't provide bit-shifting, we have to do some
  ' pretty messy string manipulation:
  ' 1. Convert the version to a hex string by using Hex()
  ' 2. Make sure the hex string is 8 characters by using Right()
  ' 3. Extract the byte we're interested in by using Left() and Mid()
  ' 4. Convert the hex byte string into a decimal string by using Val() and Str()
  TempStr = Format(Val("&H" + Left(Right("00000000" + Hex(nIpAddress), 8), 2)))
  TempStr = TempStr + "." + Format(Val("&H" + Mid(Right("00000000" + Hex(nIpAddress), 8), 3, 2)))
  TempStr = TempStr + "." + Format(Val("&H" + Mid(Right("00000000" + Hex(nIpAddress), 8), 5, 2)))
  TempStr = TempStr + "." + Format(Val("&H" + Mid(Right("00000000" + Hex(nIpAddress), 8), 7, 2)))
  
  GetIPAddressAsString = TempStr
End Function


Public Function GetResultAsString(nResult As Long) As String

    If (nResult = SIOMM_OK) Then
      GetResultAsString = "Ok"
    ElseIf (nResult = SIOMM_ERROR) Then
      GetResultAsString = "General error"
    ElseIf (nResult = SIOMM_TIME_OUT) Then
      GetResultAsString = "Timeout"
    ElseIf (nResult = SIOMM_ERROR_NO_SOCKETS) Then
      GetResultAsString = "No sockets"
    ElseIf (nResult = SIOMM_ERROR_CREATING_SOCKET) Then
      GetResultAsString = "Could not create socket to Snap I/O"
    ElseIf (nResult = SIOMM_ERROR_CONNECTING_SOCKET) Then
      GetResultAsString = "Could not connect socket to Snap I/O"
    ElseIf (nResult = SIOMM_ERROR_RESPONSE_BAD) Then
      GetResultAsString = "Bad response"
    ElseIf (nResult = SIOMM_ERROR_NOT_CONNECTED_YET) Then
      GetResultAsString = "Not connected yet"
    ElseIf (nResult = SIOMM_ERROR_OUT_OF_MEMORY) Then
      GetResultAsString = "ActiveX out of memory"
    ElseIf (nResult = SIOMM_ERROR_NOT_CONNECTED) Then
      GetResultAsString = "Not connected"
    ElseIf (nResult = SIOMM_BRAIN_ERROR_UNDEFINED_CMD) Then
      GetResultAsString = "Snap I/O Error: Undefined Command"
    ElseIf (nResult = SIOMM_BRAIN_ERROR_INVALID_PT_TYPE) Then
      GetResultAsString = "Snap I/O Error: Invalid point type"
    ElseIf (nResult = SIOMM_BRAIN_ERROR_INVALID_FLOAT) Then
      GetResultAsString = "Snap I/O Error: Invalid float"
    ElseIf (nResult = SIOMM_BRAIN_ERROR_PUC_EXPECTED) Then
      GetResultAsString = "Snap I/O Error: Powerup Clear expected"
    ElseIf (nResult = SIOMM_BRAIN_ERROR_INVALID_ADDRESS) Then
      GetResultAsString = "Snap I/O Error: Invalid memory address"
    ElseIf (nResult = SIOMM_BRAIN_ERROR_INVALID_CMD_LENGTH) Then
      GetResultAsString = "Snap I/O Error: Invalid command length"
    ElseIf (nResult = SIOMM_BRAIN_ERROR_RESERVED) Then
      GetResultAsString = "Snap I/O Error: Reserved"
    ElseIf (nResult = SIOMM_BRAIN_ERROR_BUSY) Then
      GetResultAsString = "Snap I/O Error: Busy"
    ElseIf (nResult = SIOMM_BRAIN_ERROR_CANT_ERASE_FLASH) Then
      GetResultAsString = "Snap I/O Error: Cannot erase flash"
    ElseIf (nResult = SIOMM_BRAIN_ERROR_CANT_PROG_FLASH) Then
      GetResultAsString = "Snap I/O Error: Cannot program flash"
    ElseIf (nResult = SIOMM_BRAIN_ERROR_IMAGE_TOO_SMALL) Then
      GetResultAsString = "Snap I/O Error: Downloaded imaged too small"
    ElseIf (nResult = SIOMM_BRAIN_ERROR_IMAGE_CRC_MISMATCH) Then
      GetResultAsString = "Snap I/O Error: Image CRC mismatch"
    ElseIf (nResult = SIOMM_BRAIN_ERROR_IMAGE_LEN_MISMATCH) Then
      GetResultAsString = "Snap I/O Error: Image length mismatch"
    Else
      GetResultAsString = "Unknown error"
    End If

End Function

Public Function OpenSnapIoWithCancel(Brain As OptoSnapIoMemMapX, IpAddress As String, PortNumber As Integer, Timeout As Long) As Long
  Dim nResult As Long
  
  ' Attempt to open the Snap I/O..
  nResult = Brain.OpenEnet2(IpAddress, PortNumber, Timeout, 1, SIOMM_TCP)
  
  ' Check the result
  If (nResult = SIOMM_OK) Then
    
    ' Sleep for a little while
    Sleep (250)
    
    ' Set the cancel flag to false
    dlgOpenSnapIo.bCancel = False
    
    ' Keep calling IsOpenDone() until we succeed, time-out, or error
    nResult = Brain.IsOpenDone()
    While ((nResult = SIOMM_ERROR_NOT_CONNECTED_YET) And (Not dlgOpenSnapIo.bCancel))
      
      ' let other tasks get some CPU
      DoEvents
      
      ' Show the cancel dialog
      dlgOpenSnapIo.Show (0)
      
      ' Check the status of the open operation
      nResult = Brain.IsOpenDone() ' keep trying
    Wend
    
    ' hide the cancel box
    Call dlgOpenSnapIo.Hide
    
    ' check if we should close the B3000
    If (dlgOpenSnapIo.bCancel = True) Then
      Call Brain.Close
    End If
  End If
  
  OpenSnapIoWithCancel = nResult
  
End Function

Public Function IsBitTrue(nNum As Long, nBit As Long) As Boolean
  If (nBit = 31) Then
    IsBitTrue = (nNum < 0)
  Else
    IsBitTrue = nNum And (2 ^ nBit)
  End If
End Function
