//-----------------------------------------------------------------------------
//
// O22SIOUT.h
// Copyright (c) 1999 - 2002 by Opto 22
//
// Utility header for the OptoMMP Communication Toolkit C++ classes.  
// 
// While this code was developed on Microsoft Windows 32-bit operating 
// systems, it is intended to be as generic as possible.  For Windows specific
// code, search for "_WIN32".  For Linux specific code, search for "_LINUX".
//-----------------------------------------------------------------------------

#ifndef __O22SIOUT_H_
#define __O22SIOUT_H_

#include "O22TYPES.h"

#ifdef _WIN32
#include "winsock2.h"
#include "process.h"    
#include "stdio.h"
#endif 

#ifdef _LINUX
#include <sys/time.h>
#include <sys/times.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <netinet/in.h>
#include <netdb.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <string.h>
#include <pthread.h>
#include <stdio.h>
#include <errno.h>
#include <time.h>

// The following socket items are taken from Windows
typedef int SOCKET;
#define INVALID_SOCKET 0
#define SOCKET_ERROR   -1

#endif // _LINUX


#ifdef _WIN32
#define GET_LAST_SOCKET_ERROR() WSAGetLastError()
#elif defined _LINUX
#define GET_LAST_SOCKET_ERROR() errno
#else
// For other systems, you need to provide system errors
#define GET_LAST_SOCKET_ERROR ??
#endif

// The following macros are useful for making values from smaller smarts, such as making
// a four-byte integer from the four individual bytes.

#ifndef O22MAKELONG // makes a long from 4 bytes
#define O22MAKELONG(b0, b1, b2, b3)  ((((uint32_t)(b0)) << 24) | (((uint32_t)(b1)) << 16) | (((uint32_t)(b2)) << 8) | ((uint32_t)(b3)))
#endif 

#ifndef O22MAKELONG2 // makes a long from 4 bytes
#define O22MAKELONG2(a, o)  ((((uint32_t)(a[o])) << 24) | (((uint32_t)(a[o+1])) << 16) | (((uint32_t)(a[o+2])) << 8) | ((uint32_t)(a[o+3])))
#endif 

#ifndef O22MAKEFLOAT // makes a float from 4 bytes
#define O22MAKEFLOAT(pf, b0, b1, b2, b3)  ((uint8_t*)pf)[0] = b3 ; ((uint8_t*)pf)[1] = b2 ; ((uint8_t*)pf)[2] = b1 ; ((uint8_t*)pf)[3] = b0 ;
#endif

float O22MAKEFLOAT2(uint8_t * a, int o);

#ifndef O22MAKEWORD // makes a word from 2 bytes
#define O22MAKEWORD(b0, b1)  (((uint16_t)((uint8_t)(b0))) << 8) | (uint16_t)(((uint8_t)(b1)))
#endif 

#ifndef O22MAKEWORD2 // makes a word from 2 bytes
#define O22MAKEWORD2(a, o)  (((uint16_t)((uint8_t)(a[o]))) << 8) | (uint16_t)(((uint8_t)(a[o+1])))
#endif 

#ifndef O22MAKEBYTE2 // makes a byte from 1 byte, for consistency with other macros
#define O22MAKEBYTE2(a, o)  a[o]
#endif 

#ifndef O22MAKEDWORD // makes a dwords from 2 words
#define O22MAKEDWORD(w0, w1) (((uint32_t)((uint16_t)(w0))) << 16) | (uint32_t)(((uint16_t)(w1)))
#endif

// The following macros are useful for breaking up a value into smaller pieces, such as 
// getting a third byte out of a 32-bit integer

#ifndef O22LOWORD // gets the low word from a dword
#define O22LOWORD(l) ((uint16_t)(l))
#endif

#ifndef O22HIWORD // gets the high word from a dword
#define O22HIWORD(l) ((uint16_t)(((uint32_t)(l) >> 16) & 0xFFFF))
#endif

#ifndef O22LOBYTE // gets the low byte from a word
#define O22LOBYTE(w) ((uint8_t)(w))
#endif

#ifndef O22HIBYTE // gets the high byte from a word
#define O22HIBYTE(w) ((uint8_t)(((uint16_t)(w) >> 8) & 0xFF))
#endif

#ifndef O22BYTE0 // gets the first byte from a dword
#define O22BYTE0(l) ((uint8_t)(((uint32_t)(l) >> 24) & 0xFF))
#endif

#ifndef O22BYTE1 // gets the second byte from a dword
#define O22BYTE1(l) ((uint8_t)(((uint32_t)(l) >> 16) & 0xFF))
#endif

#ifndef O22BYTE2 // gets the third byte from a dword
#define O22BYTE2(l) ((uint8_t)(((uint32_t)(l) >> 8) & 0xFF))
#endif

#ifndef O22BYTE3 // gets the fourth byte from a dword
#define O22BYTE3(l) ((uint8_t)(l))
#endif

#ifndef O22FILL_ARRAY_FROM_INT16 // Fill the given byte array with a short
#define O22FILL_ARRAY_FROM_INT16(a,i,n) a[i] = O22HIBYTE(n); a[i+1] = O22LOBYTE(n);
#endif

#ifndef O22FILL_ARRAY_FROM_INT32  // Fill the given byte array with a long
#define O22FILL_ARRAY_FROM_INT32(a,i,n)  a[i] = O22BYTE0(n); a[i+1] = O22BYTE1(n); a[i+2] = O22BYTE2(n); a[i+3] = O22BYTE3(n)
#endif

#ifndef O22FILL_ARRAY_FROM_FLOAT // Fill the given byte array with a float
#define O22FILL_ARRAY_FROM_FLOAT(a,i,f)  O22FILL_ARRAY_FROM_INT32(a, i, *((int32_t*)(&f)))
#endif

void O22FILL_ARRAY_FROM_FLOATX(uint8_t * a, int32_t i, float f);

void O22FILL_ARRAY_FROM_FLOAT_LIT(uint8_t * a, int32_t i,float f);


#ifndef O22_SWAP_BYTES_LONG // swap the bytes in a long
#define O22_SWAP_BYTES_LONG(l)  O22MAKELONG(O22BYTE3(l), O22BYTE2(l), O22BYTE1(l), O22BYTE0(l))
#endif


#define array_length( a ) (sizeof(a)/sizeof(a[0]))


// Error values generated by the classes in the communication toolkit
#define SIOMM_OK                               1
#define SIOMM_ERROR                           -1
#define SIOMM_TIME_OUT                        -2
#define SIOMM_ERROR_NO_SOCKETS                -3  // Unable to access socket interface
#define SIOMM_ERROR_CREATING_SOCKET           -4
#define SIOMM_ERROR_CONNECTING_SOCKET         -5
#define SIOMM_ERROR_RESPONSE_BAD              -6
#define SIOMM_ERROR_NOT_CONNECTED_YET         -7
#define SIOMM_ERROR_OUT_OF_MEMORY             -8
#define SIOMM_ERROR_NOT_CONNECTED             -9
#define SIOMM_ERROR_STREAM_TYPE_BAD           -10
#define SIOMM_ERROR_LENGTH_BAD                -11
#define SIOMM_ERROR_INDEX_BAD                 -12
#define SIOMM_ERROR_PARTIAL_SUCCESS           -13
#define SIOMM_ERROR_INCORRECT_LENGTH_SENT     -14
#define SIOMM_ERROR_UNEXPECTED_SELECT_RESULT  -15
#define SIOMM_ERROR_GRACEFULLY_CLOSED         -16
#define SIOMM_ERROR_WRONG_NUM_BYTES_RETURNED  -17
#define SIOMM_ERROR_UNEXPECTED_RECV_RESULT    -18
#define SIOMM_ERROR_INCORRECT_TCODE_RECIEVED  -19
#define SIOMM_ERROR_INCORRECT_TLABEL_RECIEVED -20
#define SIOMM_ERROR_UNEXPECTED_CLOSE_RESULT   -21
#define SIOMM_ERROR_INVALID_SERIAL_ADDRESS    -22
#define SIOMM_ERROR_SERIAL_PORT_IN_USE        -23
#define SIOMM_ERROR_SERIAL_MISMATCHED_CRC     -24
#define SIOMM_ERROR_SERIAL_TIMEOUT            -25
#define SIOMM_ERROR_NOT_IMPLEMENTED           -26

// Error values generated by the I/O brain itself
#define SIOMM_BRAIN_ERROR_BASE               0xE000
#define SIOMM_BRAIN_ERROR_UNDEFINED_CMD      0xE001
#define SIOMM_BRAIN_ERROR_INVALID_PT_TYPE    0xE002
#define SIOMM_BRAIN_ERROR_INVALID_FLOAT      0xE003
#define SIOMM_BRAIN_ERROR_PUC_EXPECTED       0xE004
#define SIOMM_BRAIN_ERROR_INVALID_ADDRESS    0xE005
#define SIOMM_BRAIN_ERROR_INVALID_CMD_LENGTH 0xE006
#define SIOMM_BRAIN_ERROR_RESERVED           0xE007
#define SIOMM_BRAIN_ERROR_BUSY               0xE008
#define SIOMM_BRAIN_ERROR_CANT_ERASE_FLASH   0xE009
#define SIOMM_BRAIN_ERROR_CANT_PROG_FLASH    0xE00A
#define SIOMM_BRAIN_ERROR_IMAGE_TOO_SMALL    0xE00B
#define SIOMM_BRAIN_ERROR_IMAGE_CRC_MISMATCH 0xE00C
#define SIOMM_BRAIN_ERROR_IMAGE_LEN_MISMATCH 0xE00D
#define SIOMM_BRAIN_ERROR_FEATURE_NOT_IMPL   0xE00E
#define SIOMM_BRAIN_ERROR_WATCHDOG_TIMEOUT   0xE00F

// The types of socket connections
#define SIOMM_TCP   SOCK_STREAM
#define SIOMM_UDP   SOCK_DGRAM

// Helper to calculate the CRC for serial transfers.
uint16_t Crc16R(const uint8_t *pu8aryStr, uint32_t dwLength);

const uint16_t waryCrc16r_Table [256] = 
{
  0x0000, 0xC0C1, 0xC181, 0x0140, 0xC301, 0x03C0, 0x0280, 0xC241, 0xC601, 0x06C0, 0x0780, 0xC741, 0x0500, 0xC5C1, 0xC481, 0x0440,
  0xCC01, 0x0CC0, 0x0D80, 0xCD41, 0x0F00, 0xCFC1, 0xCE81, 0x0E40, 0x0A00, 0xCAC1, 0xCB81, 0x0B40, 0xC901, 0x09C0, 0x0880, 0xC841,
  0xD801, 0x18C0, 0x1980, 0xD941, 0x1B00, 0xDBC1, 0xDA81, 0x1A40, 0x1E00, 0xDEC1, 0xDF81, 0x1F40, 0xDD01, 0x1DC0, 0x1C80, 0xDC41,
  0x1400, 0xD4C1, 0xD581, 0x1540, 0xD701, 0x17C0, 0x1680, 0xD641, 0xD201, 0x12C0, 0x1380, 0xD341, 0x1100, 0xD1C1, 0xD081, 0x1040,
  0xF001, 0x30C0, 0x3180, 0xF141, 0x3300, 0xF3C1, 0xF281, 0x3240, 0x3600, 0xF6C1, 0xF781, 0x3740, 0xF501, 0x35C0, 0x3480, 0xF441,
  0x3C00, 0xFCC1, 0xFD81, 0x3D40, 0xFF01, 0x3FC0, 0x3E80, 0xFE41, 0xFA01, 0x3AC0, 0x3B80, 0xFB41, 0x3900, 0xF9C1, 0xF881, 0x3840,
  0x2800, 0xE8C1, 0xE981, 0x2940, 0xEB01, 0x2BC0, 0x2A80, 0xEA41, 0xEE01, 0x2EC0, 0x2F80, 0xEF41, 0x2D00, 0xEDC1, 0xEC81, 0x2C40,
  0xE401, 0x24C0, 0x2580, 0xE541, 0x2700, 0xE7C1, 0xE681, 0x2640, 0x2200, 0xE2C1, 0xE381, 0x2340, 0xE101, 0x21C0, 0x2080, 0xE041,
  0xA001, 0x60C0, 0x6180, 0xA141, 0x6300, 0xA3C1, 0xA281, 0x6240, 0x6600, 0xA6C1, 0xA781, 0x6740, 0xA501, 0x65C0, 0x6480, 0xA441,
  0x6C00, 0xACC1, 0xAD81, 0x6D40, 0xAF01, 0x6FC0, 0x6E80, 0xAE41, 0xAA01, 0x6AC0, 0x6B80, 0xAB41, 0x6900, 0xA9C1, 0xA881, 0x6840,
  0x7800, 0xB8C1, 0xB981, 0x7940, 0xBB01, 0x7BC0, 0x7A80, 0xBA41, 0xBE01, 0x7EC0, 0x7F80, 0xBF41, 0x7D00, 0xBDC1, 0xBC81, 0x7C40,
  0xB401, 0x74C0, 0x7580, 0xB541, 0x7700, 0xB7C1, 0xB681, 0x7640, 0x7200, 0xB2C1, 0xB381, 0x7340, 0xB101, 0x71C0, 0x7080, 0xB041,
  0x5000, 0x90C1, 0x9181, 0x5140, 0x9301, 0x53C0, 0x5280, 0x9241, 0x9601, 0x56C0, 0x5780, 0x9741, 0x5500, 0x95C1, 0x9481, 0x5440,
  0x9C01, 0x5CC0, 0x5D80, 0x9D41, 0x5F00, 0x9FC1, 0x9E81, 0x5E40, 0x5A00, 0x9AC1, 0x9B81, 0x5B40, 0x9901, 0x59C0, 0x5880, 0x9841,
  0x8801, 0x48C0, 0x4980, 0x8941, 0x4B00, 0x8BC1, 0x8A81, 0x4A40, 0x4E00, 0x8EC1, 0x8F81, 0x4F40, 0x8D01, 0x4DC0, 0x4C80, 0x8C41,
  0x4400, 0x84C1, 0x8581, 0x4540, 0x8701, 0x47C0, 0x4680, 0x8641, 0x8201, 0x42C0, 0x4380, 0x8341, 0x4100, 0x81C1, 0x8081, 0x4040
};


// Serial pass through memory map area
#define MEMORY_SERIALPASSTHRU_CFG0   0xF0329000 // Serial Pass Through Config (Port 0).
#define MEMORY_SERIALPASSTHRU_RW0    0xF0329100 // Serial Pass Through R/W Buffer (Port 0).
#define MEMORY_SERIALPASSTHRU_CFG1   0xF032A000 // Serial Pass Through Config (Port 1).
#define MEMORY_SERIALPASSTHRU_RW1    0xF032A100 // Serial Pass Through R/W Buffer (Port 1).
#define MEMORY_SERIALPASSTHRU_CFG2   0xF032B000 // Serial Pass Through Config (Port 2).
#define MEMORY_SERIALPASSTHRU_RW2    0xF032B100 // Serial Pass Through R/W Buffer (Port 2).
#define MEMORY_SERIALPASSTHRU_CFG3   0xF032C000 // Serial Pass Through Config (Port 3).
#define MEMORY_SERIALPASSTHRU_RW3    0xF032C100 // Serial Pass Through R/W Buffer (Port 3).
#define MEMORY_SERIALPASSTHRU_CFG4   0xF032D000 // Serial Pass Through Config (Port 4).
#define MEMORY_SERIALPASSTHRU_RW4    0xF032D100 // Serial Pass Through R/W Buffer (Port 4).
#define MEMORY_SERIALPASSTHRU_CFG5   0xF032E000 // Serial Pass Through Config (Port 5).
#define MEMORY_SERIALPASSTHRU_RW5    0xF032E100 // Serial Pass Through R/W Buffer (Port 5).

// Transaction code used by the I/O unit
#define SIOMM_TCODE_WRITE_QUAD_REQUEST   0
#define SIOMM_TCODE_WRITE_BLOCK_REQUEST  1
#define SIOMM_TCODE_WRITE_RESPONSE       2
#define SIOMM_TCODE_READ_QUAD_REQUEST    4
#define SIOMM_TCODE_READ_BLOCK_REQUEST   5
#define SIOMM_TCODE_READ_QUAD_RESPONSE   6
#define SIOMM_TCODE_READ_BLOCK_RESPONSE  7

// Sizes of the read/write requests and responses
#define SIOMM_SIZE_WRITE_QUAD_REQUEST    16   
#define SIOMM_SIZE_WRITE_BLOCK_REQUEST   16   
#define SIOMM_SIZE_WRITE_RESPONSE        12
#define SIOMM_SIZE_READ_QUAD_REQUEST     12
#define SIOMM_SIZE_READ_QUAD_RESPONSE    16
#define SIOMM_SIZE_READ_BLOCK_REQUEST    16
#define SIOMM_SIZE_READ_BLOCK_RESPONSE   16

// Response codes from the I/O unit
#define SIOMM_RESPONSE_CODE_ACK          0
#define SIOMM_RESPONSE_CODE_NAK          7

// Memory Map values

// Status read area of the memory map
#define SIOMM_STATUS_READ_BASE                 0xF0300000
#define SIOMM_STATUS_READ_PUC_FLAG             0xF0300004
#define SIOMM_STATUS_READ_LAST_ERROR           0xF030000C
#define SIOMM_STATUS_READ_BOOTP_FLAG           0xF0300048
#define SIOMM_STATUS_READ_DEGREES_FLAG         0xF030004C
#define SIOMM_STATUS_READ_WATCHDOG_TIME        0xF0300054
#define SIOMM_STATUS_READ_PART_NUMBER          0xF0300080
#define SIOMM_STATUS_READ_PART_NUMBER_SIZE     0x00000020

// Date and Time
#define SIOMM_DATE_AND_TIME_BASE               0xF0350000
#define SIOMM_DATE_AND_TIME_SIZE               0x00000017


// Serial Module
#define SIOMM_SERIAL_MODULE_CONFIG_BASE        0xF03A8000
#define SIOMM_SERIAL_MODULE_CONFIG_BOUNDARY    0x00000010
#define SIOMM_SERIAL_MODULE_EOM_BASE           0xF03A8200
#define SIOMM_SERIAL_MODULE_EOM_BOUNDARY       0x00000010

// Status write area of the memory map
#define SIOMM_STATUS_WRITE_OPERATION           0xF0380000
#define SIOMM_STATUS_WRITE_BASE                0xF0380004
#define SIOMM_STATUS_WRITE_BOOTP               0xF0380004
#define SIOMM_STATUS_WRITE_TEMP_DEGREES        0xF0380008
#define SIOMM_STATUS_WRITE_WATCHDOG_TIME       0xF0380010

// Streaming configuration area of the memory map
#define SIOMM_STREAM_CONFIG_BASE               0xF03FFFC4
#define SIOMM_STREAM_CONFIG_ON_FLAG            0xF03FFFD0
#define SIOMM_STREAM_CONFIG_INTERVAL           0xF03FFFD4
#define SIOMM_STREAM_CONFIG_PORT               0xF03FFFD8

#define SIOMM_STREAM_TARGET_BOUNDARY           0x00000004
#define SIOMM_STREAM_TARGET_BASE               0xF03FFFE0

// Digital bank read area of the memory map
#define SIOMM_DBANK_READ_AREA_BASE             0xF0400000
#define SIOMM_DBANK_READ_POINT_STATES          0xF0400000
#define SIOMM_DBANK_READ_ON_LATCH_STATES       0xF0400008
#define SIOMM_DBANK_READ_OFF_LATCH_STATES      0xF0400010
#define SIOMM_DBANK_READ_ACTIVE_COUNTERS       0xF0400018
#define SIOMM_DBANK_READ_COUNTER_DATA_BASE     0xF0400100
#define SIOMM_DBANK_READ_COUNTER_DATA_BOUNDARY 0x00000004
#define SIOMM_DBANK_MAX_FEATURE_ELEMENTS       0x00000040

// Digital bank write area of the memory map
#define SIOMM_DBANK_WRITE_AREA_BASE            0xF0500000
#define SIOMM_DBANK_WRITE_TURN_ON_MASK         0xF0500000
#define SIOMM_DBANK_WRITE_TURN_OFF_MASK        0xF0500008
#define SIOMM_DBANK_WRITE_ACT_COUNTERS_MASK    0xF0500010
#define SIOMM_DBANK_WRITE_DEACT_COUNTERS_MASK  0xF0500018

// Analog bank read area of the memory map
#define SIOMM_ABANK_READ_AREA_BASE             0xF0600000
#define SIOMM_ABANK_READ_POINT_VALUES          0xF0600000
#define SIOMM_ABANK_READ_POINT_COUNTS          0xF0600100
#define SIOMM_ABANK_READ_POINT_MIN_VALUES      0xF0600200
#define SIOMM_ABANK_READ_POINT_MAX_VALUES      0xF0600300

// Analog bank write area of the memory map
#define SIOMM_ABANK_WRITE_AREA_BASE            0xF0700000
#define SIOMM_ABANK_WRITE_POINT_VALUES         0xF0700000
#define SIOMM_ABANK_WRITE_POINT_COUNTS         0xF0700100

#define SIOMM_ABANK_MAX_BYTES                  0x00000100
#define SIOMM_ABANK_MAX_ELEMENTS               0x00000040


// Digital point read area of the memory map
#define SIOMM_DPOINT_READ_BOUNDARY             0x00000040
#define SIOMM_DPOINT_READ_AREA_BASE            0xF0800000
#define SIOMM_DPOINT_READ_STATE                0xF0800000
#define SIOMM_DPOINT_READ_ONLATCH_STATE        0xF0800004
#define SIOMM_DPOINT_READ_OFFLATCH_STATE       0xF0800008
#define SIOMM_DPOINT_READ_ACTIVE_COUNTER       0xF080000C
#define SIOMM_DPOINT_READ_COUNTER_DATA         0xF0800010

// Digital point write area of the memory map
#define SIOMM_DPOINT_WRITE_BOUNDARY            0x00000040
#define SIOMM_DPOINT_WRITE_TURN_ON_BASE        0xF0900000
#define SIOMM_DPOINT_WRITE_TURN_OFF_BASE       0xF0900004
#define SIOMM_DPOINT_WRITE_ACTIVATE_COUNTER    0xF0900008
#define SIOMM_DPOINT_WRITE_DEACTIVATE_COUNTER  0xF090000C

// Analog point read area of the memory map
#define SIOMM_APOINT_READ_BOUNDARY             0x00000040
#define SIOMM_APOINT_READ_AREA_BASE            0xF0A00000
#define SIOMM_APOINT_READ_VALUE_BASE           0xF0A00000
#define SIOMM_APOINT_READ_COUNTS_BASE          0xF0A00004
#define SIOMM_APOINT_READ_MIN_VALUE_BASE       0xF0A00008
#define SIOMM_APOINT_READ_MAX_VALUE_BASE       0xF0A0000C
#define SIOMM_APOINT_READ_TPO_PERIOD_BASE      0xF0B0000C

// Analog point write area of the memory map
#define SIOMM_APOINT_WRITE_BOUNDARY            0x00000040
#define SIOMM_APOINT_WRITE_VALUE_BASE          0xF0B00000
#define SIOMM_APOINT_WRITE_COUNTS_BASE         0xF0B00004
#define SIOMM_APOINT_WRITE_TPO_PERIOD_BASE     0xF0B0000C

// Point configuration area of the memory map
#define SIOMM_POINT_CONFIG_BOUNDARY               0x00000040
#define SIOMM_POINT_CONFIG_READ_MOD_TYPE_BASE     0xF0C00000
#define SIOMM_POINT_CONFIG_WRITE_TYPE_BASE        0xF0C00004
#define SIOMM_POINT_CONFIG_WRITE_FEATURE_BASE     0xF0C00008
#define SIOMM_POINT_CONFIG_WRITE_OFFSET_BASE      0xF0C0000C
#define SIOMM_POINT_CONFIG_WRITE_GAIN_BASE        0xF0C00010
#define SIOMM_POINT_CONFIG_WRITE_HISCALE_BASE     0xF0C00014
#define SIOMM_POINT_CONFIG_WRITE_LOSCALE_BASE     0xF0C00018
#define SIOMM_POINT_CONFIG_WRITE_FILTER_BASE      0xF0C00020
#define SIOMM_POINT_CONFIG_WRITE_WDOG_VALUE_BASE  0xF0C00024
#define SIOMM_POINT_CONFIG_WRITE_WDOG_ENABLE_BASE 0xF0C00028
#define SIOMM_POINT_CONFIG_WRITE_NAME_BASE        0xF0C00030
#define SIOMM_POINT_CONFIG_NAME_SIZE              0x00000010
#define SIOMM_POINT_CONFIG_NAME_SIZE_NO_NULL      0x0000000F
#define SIOMM_POINT_CONFIG_CLAMP_BOUNDARY         0x00000008
#define SIOMM_POINT_CONFIG_WRITE_LOCLAMP_BASE     0xF0C01000
#define SIOMM_POINT_CONFIG_WRITE_HICLAMP_BASE     0xF0C01004

#define SIOMM_PIDLOOP_CONFIG_BOUNDARY             0x00000080
#define SIOMM_PIDLOOP_CONFIG_BASE                 0xF2100000
#define SIOMM_PIDLOOP_CONFIG_GAIN_BASE            0xF2100010

#define SIOMM_TPO_CONFIG_BOUNDARY                 0x00000030
#define SIOMM_TPO_CONFIG_MEMMAP_BASE              0xF08C4000
#define SIOMM_TPO_CONFIG_OUTPUT_MEMMAP_BASE       0xF08C401C
#define SIOMM_TPO_ON_MASK_BASE                    0XF08C0040
#define SIOMM_TPO_OFF_MASK_BASE                   0XF08C0080

#define SIOMM_EVENT_MSG_BOUNDARY                  0x000000C0
#define SIOMM_EVENT_MSG_MEMMAP_BASE               0xF1200000
#define SIOMM_EVENT_MSG_MSG_LENGTH                0x00000080

#define SIOMM_DIGITAL_EVENT_BOUNDARY              0x00000040
#define SIOMM_DIGITAL_EVENT_MEMMAP_BASE           0xF0D00000

#define SIOMM_ALARM_EVENT_BOUNDARY                0x00000080
#define SIOMM_ALARM_EVENT_MEMMAP_BASE             0xF1100000

#define SIOMM_SERIAL_EVENT_BOUNDARY               0x00000074
#define SIOMM_SERIAL_EVENT_MEMMAP_BASE            0xF1540000

#define SIOMM_TIMER_EVENT_BOUNDARY                0x00000080
#define SIOMM_TIMER_EVENT_MEMMAP_BASE             0xF0D40000
#define SIOMM_TIMER_EVENT_MEMMAP_END              0xF0D4FFFF


// Digital point read and clear area of the memory map
#define SIOMM_DPOINT_READ_CLEAR_BOUNDARY       0x00000004
#define SIOMM_DPOINT_READ_CLEAR_COUNTS_BASE    0xF0F00000
#define SIOMM_DPOINT_READ_CLEAR_ON_LATCH_BASE  0xF0F00100
#define SIOMM_DPOINT_READ_CLEAR_OFF_LATCH_BASE 0xF0F00200

// Analog point read and clear area of the memory map
#define SIOMM_APOINT_READ_CLEAR_BOUNDARY       0x00000004
#define SIOMM_APOINT_READ_CLEAR_MIN_VALUE_BASE 0xF0F80000
#define SIOMM_APOINT_READ_CLEAR_MAX_VALUE_BASE 0xF0F80100

// Analog point calc and set area of the memory map
#define SIOMM_APOINT_READ_CALC_SET_BOUNDARY    0x00000004
#define SIOMM_APOINT_READ_CALC_SET_OFFSET_BASE 0xF0E00000
#define SIOMM_APOINT_READ_CALC_SET_GAIN_BASE   0xF0E00100

// Streaming read area of the memory map
#define SIOMM_STREAM_READ_AREA_BASE            0xF1000000
#define SIOMM_STREAM_READ_AREA_SIZE            0x00000220

// Scratch pad aread of the memory map
#define SIOMM_SCRATCHPAD_BITS_BASE             0xF0D80000
#define SIOMM_SCRATCHPAD_BITS_ON_MASK_BASE     0xF0D80400
#define SIOMM_SCRATCHPAD_BITS_OFF_MASK_BASE    0xF0D80408
#define SIOMM_SCRATCHPAD_INTEGER_BOUNDARY      0x00000004
#define SIOMM_SCRATCHPAD_INTEGER_BASE          0xF0D81000
#define SIOMM_SCRATCHPAD_INTEGER_BASE1         0xF0D81000
#define SIOMM_SCRATCHPAD_INTEGER_BASE2         0xF0DA0000
#define SIOMM_SCRATCHPAD_INTEGER_BASE3         0xF0DA2000
#define SIOMM_SCRATCHPAD_INTEGER_MAX_ELEMENTS1 0x00000400
#define SIOMM_SCRATCHPAD_INTEGER_MAX_ELEMENTS2 0x00000800
#define SIOMM_SCRATCHPAD_INTEGER_MAX_ELEMENTS3 0x00001C00
#define SIOMM_SCRATCHPAD_INTEGER_MAX_ELEMENTS    \
        SIOMM_SCRATCHPAD_INTEGER_MAX_ELEMENTS1 + \
        SIOMM_SCRATCHPAD_INTEGER_MAX_ELEMENTS2 + \
        SIOMM_SCRATCHPAD_INTEGER_MAX_ELEMENTS3
#define SIOMM_SCRATCHPAD_INTEGER_MAX_BYTES       \
        SIOMM_SCRATCHPAD_INTEGER_MAX_ELEMENTS * 4
#define SIOMM_SCRATCHPAD_FLOAT_BOUNDARY        0x00000004
#define SIOMM_SCRATCHPAD_FLOAT_BASE            0xF0D82000
#define SIOMM_SCRATCHPAD_FLOAT_BASE1           0xF0D82000
#define SIOMM_SCRATCHPAD_FLOAT_BASE2           0xF0DC0000
#define SIOMM_SCRATCHPAD_FLOAT_BASE3           0xF0DC2000
#define SIOMM_SCRATCHPAD_FLOAT_MAX_ELEMENTS1   0x00000400
#define SIOMM_SCRATCHPAD_FLOAT_MAX_ELEMENTS2   0x00000800
#define SIOMM_SCRATCHPAD_FLOAT_MAX_ELEMENTS3   0x00001C00
#define SIOMM_SCRATCHPAD_FLOAT_MAX_ELEMENTS      \
        SIOMM_SCRATCHPAD_FLOAT_MAX_ELEMENTS1 +   \
        SIOMM_SCRATCHPAD_FLOAT_MAX_ELEMENTS2 +   \
        SIOMM_SCRATCHPAD_FLOAT_MAX_ELEMENTS3
#define SIOMM_SCRATCHPAD_FLOAT_MAX_BYTES         \
        SIOMM_SCRATCHPAD_FLOAT_MAX_ELEMENTS * 4
#define SIOMM_SCRATCHPAD_STRING_BASE           0xF0D83000
#define SIOMM_SCRATCHPAD_STRING_LENGTH_BASE    0xF0D83000
#define SIOMM_SCRATCHPAD_STRING_LENGTH_SIZE    0x00000002
#define SIOMM_SCRATCHPAD_STRING_DATA_BASE      0xF0D83002
#define SIOMM_SCRATCHPAD_STRING_DATA_SIZE      0x00000080
#define SIOMM_SCRATCHPAD_STRING_BOUNDARY       0x00000082
#define SIOMM_SCRATCHPAD_STRING_MAX_BYTES      0x00000410
#define SIOMM_SCRATCHPAD_STRING_MAX_ELEMENTS   0x00000008
#define SIOMM_SCRATCHPAD_INTEGER_64_BASE       0xF0DE0000
#define SIOMM_SCRATCHPAD_INTEGER_64_MAX_ELEMENTS 0x00000400

// Read Block as type
#define SIOMM_BLOCK_MAX_BYTES                  0x00000400
#define SIOMM_BLOCK_AS_INT_MAX_ELEMENTS        0x00004000 // Have a limit just 
#define SIOMM_BLOCK_AS_FLT_MAX_ELEMENTS        0x00004000 // to keep things 
                                                          // reasonable.

// Data Logging Samples read area of the memory map
#define SIOMM_DATALOG_SAMPLES_BASE             0xF3020000
#define SIOMM_DATALOG_SAMPLES_BOUNDARY         0x00000014
#define SIOMM_DATALOG_SAMPLES_READ_BLOCK_COUNT 50
#define SIOMM_DATALOG_SAMPLES_READ_BLOCK_SIZE    \
        SIOMM_DATALOG_SAMPLES_BOUNDARY *         \
        SIOMM_DATALOG_SAMPLES_READ_BLOCK_COUNT
#define SIOMM_DATALOG_SAMPLES_ORIG_LENGTH      300 // The original size
                                                   // of the datalog.
#define SIOMM_DATALOG_SAMPLES_MEMMAP_OFFSET    8
#define SIOMM_DATALOG_SAMPLES_FORMAT_OFFSET    12
#define SIOMM_DATALOG_SAMPLES_DATA_OFFSET      16


// Digital point read area of the memory map
#define SIOMM4096_DPOINT_READ_BOUNDARY             0x00000040
#define SIOMM4096_DPOINT_READ_AREA_BASE            0xF1808000
#define SIOMM4096_DPOINT_READ_STATE                0xF1808000
#define SIOMM4096_DPOINT_READ_ONLATCH_STATE        0xF1808008
#define SIOMM4096_DPOINT_READ_OFFLATCH_STATE       0xF1808010
#define SIOMM4096_DPOINT_READ_COUNTER_BOUNDARY     0x00000100
#define SIOMM4096_DPOINT_READ_COUNTER_DATA         0xF1809000

// Digital point write area of the memory map
#define SIOMM4096_DPOINT_WRITE_BOUNDARY            0x00000040
#define SIOMM4096_DPOINT_WRITE_TURN_ON_BASE        0xF180C000
#define SIOMM4096_DPOINT_WRITE_TURN_OFF_BASE       0xF180C008

// Digital point read/clear area of the memory map
#define SIOMM4096_DPOINT_READCLEAR_LATCH_BOUNDARY  0x00000020
#define SIOMM4096_DPOINT_READCLEAR_ONLATCH         0xF180A000
#define SIOMM4096_DPOINT_READCLEAR_OFFLATCH        0xF180A008
#define SIOMM4096_DPOINT_READCLEAR_COUNTER_BOUNDARY 0x00000100
#define SIOMM4096_DPOINT_READCLEAR_COUNTER         0xF180B000

// Analog point read area of the memory map
#define SIOMM4096_APOINT_READ_BOUNDARY             0x00000040
#define SIOMM4096_APOINT_READ_POINT_OFFSET         0x00000040
#define SIOMM4096_APOINT_READ_MODULE_OFFSET        0x00001000
#define SIOMM4096_APOINT_READ_AREA_BASE            0xF0260000
#define SIOMM4096_APOINT_READ_VALUE_BASE           0xF0260000
#define SIOMM4096_APOINT_READ_COUNTS_BASE          0xF0260004
#define SIOMM4096_APOINT_READ_MIN_VALUE_BASE       0xF0260008
#define SIOMM4096_APOINT_READ_MAX_VALUE_BASE       0xF026000C
#define SIOMM4096_APOINT_READ_RAW_COUNTS_BASE      0xF0260024
#define SIOMM4096_APOINT_READ_TPO_PERIOD_BASE      0xF02A000C

// Analog point write area of the memory map
#define SIOMM4096_APOINT_WRITE_BOUNDARY            0x00000040
#define SIOMM4096_APOINT_WRITE_POINT_OFFSET        0x00000040
#define SIOMM4096_APOINT_WRITE_MODULE_OFFSET       0x00001000
#define SIOMM4096_APOINT_WRITE_AREA_BASE           0xF02A0000
#define SIOMM4096_APOINT_WRITE_VALUE_BASE          0xF02A0000
#define SIOMM4096_APOINT_WRITE_COUNTS_BASE         0xF02A0004
#define SIOMM4096_APOINT_WRITE_TPO_PERIOD_BASE     0xF02A000C

// Point configuration area of the memory map
#define SIOMM4096_POINT_CONFIG_BOUNDARY               0x000000C0
#define SIOMM4096_POINT_CONFIG_AREA_BASE              0xF0100000
#define SIOMM4096_POINT_CONFIG_READ_MOD_TYPE_BASE     0xF0100000
#define SIOMM4096_POINT_CONFIG_WRITE_TYPE_BASE        0xF0100004
#define SIOMM4096_POINT_CONFIG_WRITE_FEATURE_BASE     0xF0100008
#define SIOMM4096_POINT_CONFIG_WRITE_OFFSET_BASE      0xF010000C
#define SIOMM4096_POINT_CONFIG_WRITE_GAIN_BASE        0xF0100010
#define SIOMM4096_POINT_CONFIG_WRITE_HISCALE_BASE     0xF0100014
#define SIOMM4096_POINT_CONFIG_WRITE_LOSCALE_BASE     0xF0100018
#define SIOMM4096_POINT_CONFIG_WRITE_FILTER_BASE      0xF0100020
#define SIOMM4096_POINT_CONFIG_WRITE_WDOG_VALUE_BASE  0xF0100024
#define SIOMM4096_POINT_CONFIG_WRITE_WDOG_ENABLE_BASE 0xF0100028
#define SIOMM4096_POINT_CONFIG_WRITE_NAME_BASE        0xF0100030
#define SIOMM4096_POINT_CONFIG_NAME_SIZE              0x00000033
#define SIOMM4096_POINT_CONFIG_NAME_SIZE_NO_NULL      0x00000032
#define SIOMM4096_POINT_CONFIG_WRITE_LOCLAMP_BASE     0xF01000B8
#define SIOMM4096_POINT_CONFIG_WRITE_HICLAMP_BASE     0xF01000BC

// Digital point read and clear area of the memory map
#define SIOMM4096_DPOINT_READ_CLEAR_BOUNDARY       0x0000000C
#define SIOMM4096_DPOINT_READ_CLEAR_COUNTS_BASE    0xF01C8000
#define SIOMM4096_DPOINT_READ_CLEAR_ON_LATCH_BASE  0xF01C8004
#define SIOMM4096_DPOINT_READ_CLEAR_OFF_LATCH_BASE 0xF01C8008

// Analog point read and clear area of the memory map
#define SIOMM4096_APOINT_READ_CLEAR_BOUNDARY       0x0000000C
#define SIOMM4096_APOINT_READ_CLEAR_BASE           0xF01D4000
#define SIOMM4096_APOINT_READ_CLEAR_POINT_OFFSET   0x0000000C
#define SIOMM4096_APOINT_READ_CLEAR_MODULE_OFFSET  0x00000300
#define SIOMM4096_APOINT_READ_CLEAR_MIN_VALUE_BASE 0xF01D4000
#define SIOMM4096_APOINT_READ_CLEAR_MAX_VALUE_BASE 0xF01D4004

// Analog point calc and set area of the memory map
#define SIOMM4096_APOINT_READ_CALC_SET_BOUNDARY    0x00000008
#define SIOMM4096_APOINT_READ_CALC_SET_OFFSET_BASE 0xF01C0000
#define SIOMM4096_APOINT_READ_CALC_SET_GAIN_BASE   0xF01C0004

// Maximum number of expanded points and modules; used to make sure
// read/write requests don't leave the designated memory space
#define SIOMM4096_MAX_POINTS                       64
#define SIOMM4096_MAX_MODULES                      64

// Helpers to get the memory addresses for serial passthru
unsigned int GetPassThruConfigAddress(int port);
unsigned int GetPassThruReadWriteAddress(int port);

#endif // __O22SIOUT_H_

